@@ -9,14 +9,15 @@ import numpy as np
 import cv2 as cv
 import io
 from threading import Thread
-#import keyboard
+import keyboard
 import time
+from Cogenvdecoder import  UdpComms as U
 
 num_vector = 7
 
 
 class CogEnvDecoder:
-    def __init__(self, size=(400, 400), train=True, allow_multiple_obs=True, env_name='RealGame.exe'):
+    def __init__(self, size=(400, 400), train=True, allow_multiple_obs=True, env_name='RealGame.exe', mat_num=0):
         if train:
             engine_configuration_channel = EngineConfigurationChannel()
             engine_configuration_channel.set_configuration_parameters(width=100, height=100, time_scale=100)
@@ -32,7 +33,14 @@ class CogEnvDecoder:
         self.metadata = self._env.metadata
         self.allow_multiple_obs = allow_multiple_obs
         self.spec = self._env.spec
-
+        self._end_thread = False
+        self._sock = U.UdpComms(udpIP="127.0.0.1", portTX=8000, portRX=8001, enableRX=True, suppressWarnings=True)
+        self.depth_sock = U.UdpComms(udpIP="127.0.0.1", portTX=8002, portRX=8003,enableRX=True, suppressWarnings=True)
+        self.last_frame = np.zeros((100, 100), dtype=np.uint8)
+        self.last_depth_frame = np.zeros((100, 100), dtype=np.uint8)
+        data = open("RealGame_Data/data.txt", "w")
+        data.write(str(mat_num))
+        data.close()
 
     @property
     def observation_space(self):
@@ -50,8 +58,12 @@ class CogEnvDecoder:
         return self._env.close()
 
     def reset(self):
+        #self._end_thread = True
+        #self._thread.join()
         obs = self._env.reset()
-        img = self._GetImg(obs)
+        #self._end_thread = False
+        #self.step_thread_begin()
+        img = self.read_img()
         self_pos = self._GetSelfPos(obs)  # Blue one Position
         enemy_pos = self._EnemyPos(obs)  # Red one position
         goal1 = self._GetGoal1Pos(obs)  # Goal 1 position, whether it has been activated
@@ -80,9 +92,11 @@ class CogEnvDecoder:
             return self._env._single_step(decision_step)
 
     def step_action(self, action):
-        obs, reward, done, info = self.action_transfer(action)
+        obs, reward, done, info = self._env.step(action)
         # obs = self._get_obs(obs)
-        img = self._GetImg(obs)
+
+        img = self.read_img()
+        depth_img = self.read_depth_img()
         self_pos = self._GetSelfPos(obs)  # Blue one Position
         enemy_pos = self._EnemyPos(obs)  # Red one position
         goal1 = self._GetGoal1Pos(obs)  # Goal 1 position, whether it has been activated
@@ -98,13 +112,29 @@ class CogEnvDecoder:
         dmg = self._DmgCaused(obs)  # blue one caused damage to red one
         time = self._TimeTaken(obs)  # Time passed in the round
         collision_info = self._CollCondtion(obs)  # collision times and continuous collision time
-        final_obs = [img, self_pos, enemy_pos, self_info, goal1, goal2, goal3, goal4, goal5, collision_info]
+        final_obs = [img, self_pos, enemy_pos, self_info, goal1, goal2, goal3, goal4, goal5, collision_info, depth_img]
         judge_result = [score, flag_ach, enemy_act, enemy_info, dmg, time]
         return final_obs, reward, done, [info, judge_result]
 
     def render(self, mode):
         return self._env.render(mode)
 
+    def read_img(self):
+        img = self._sock.ReadReceivedData()
+        if not isinstance(img, np.ndarray):
+            img = self.last_frame
+        else:
+            self.last_frame = img
+        return img
+
+    def read_depth_img(self):
+        depth_img = self.depth_sock.ReadReceivedData()
+        if not isinstance(depth_img, np.ndarray):
+            depth_img = self.last_depth_frame
+        else:
+            self.last_depth_frame = depth_img
+        return depth_img
+
     def _get_obs(self, obs):
         img = np.zeros((400, 400, 3), np.uint8)
         # end_position = (math.floor(obs[0]*100),math.floor(obs[1]*100))
@@ -174,99 +204,98 @@ class CogEnvDecoder:
             return gray0
 
     def _GetSelfPos(self, data):
-        xPos = data[100001]
-        zPos = data[100002]
-        angle = data[100003]
+        xPos = data[0]
+        zPos = data[1]
+        angle = data[2]
         return [xPos, zPos, angle]
 
     def _GetGoal1Pos(self, data):
-        xPos = data[100004]
-        zPos = data[100005]
-        if data[100014] >= 1:
+        xPos = data[3]
+        zPos = data[4]
+        if data[13] >= 1:
             return [xPos, zPos, True]
         else:
             return [xPos, zPos, False]
 
     def _GetGoal2Pos(self, data):
-        xPos = data[100006]
-        zPos = data[100007]
-        if data[100014] >= 2:
+        xPos = data[5]
+        zPos = data[6]
+        if data[13] >= 2:
             return [xPos, zPos, True]
         else:
             return [xPos, zPos, False]
 
     def _GetGoal3Pos(self, data):
-        xPos = data[100008]
-        zPos = data[100009]
-        if data[100014] >= 3:
+        xPos = data[7]
+        zPos = data[8]
+        if data[13] >= 3:
             return [xPos, zPos, True]
         else:
             return [xPos, zPos, False]
 
     def _GetGoal4Pos(self, data):
-        xPos = data[100010]
-        zPos = data[100011]
-        if data[100014] >= 4:
+        xPos = data[9]
+        zPos = data[10]
+        if data[13] >= 4:
             return [xPos, zPos, True]
         else:
             return [xPos, zPos, False]
 
     def _GetGoal5Pos(self, data):
-        xPos = data[100012]
-        zPos = data[100013]
-        if data[100014] > 4:
+        xPos = data[11]
+        zPos = data[12]
+        if data[13] > 4:
             return [xPos, zPos, True]
         else:
             return [xPos, zPos, False]
 
     def _AchievedGoals(self, data):
-        return data[100014]
+        return data[13]
 
     def _EnemyStatus(self, data):
-        return (data[100014] == 5)
+        return (data[13] == 5)
 
     def _EnemyPos(self, data):
-        xPos = data[100015]
-        zPos = data[100016]
-        angle = data[100017]
+        xPos = data[14]
+        zPos = data[15]
+        angle = data[16]
         return [xPos, zPos, angle]
 
     def _EnemyInfo(self, data):
-        HP = data[100018]
-        Bullet = data[100019]
+        HP = data[17]
+        Bullet = data[18]
         return [HP, Bullet]
 
     def _Score(self, data):
-        return data[100020]
+        return data[19]
 
     def _SelfInfo(self, data):
-        HP = data[100021]
-        Bullet = data[100026]
+        HP = data[20]
+        Bullet = data[25]
         return [HP, Bullet]
 
     def _DmgCaused(self, data):
-        dmg = data[100022]
+        dmg = data[21]
         return dmg
 
     def _TimeTaken(self, data):
-        time = data[100023]
+        time = data[22]
         return time
 
     def _CollCondtion(self, data):
-        coll_time = data[100024]
-        cont_coll_time = data[100025]
+        coll_time = data[23]
+        cont_coll_time = data[24]
         return [coll_time, cont_coll_time]
-
+'''
     def _thread_step(self):
-        #quit = False
+        quit = False
         #print('1')
         while True:
             self._env._env.step()
-            time.sleep(0.015)
-            #if keyboard.is_pressed('q'):
-            #    quit = True
-            #    self.close()
-            #    self._thread.join()
+            time.sleep(0.02)
+            if self._end_thread == True:
+                return
+
 
 
 
@@ -278,3 +307,4 @@ class CogEnvDecoder:
         #self._thread.join()
         #print('4')
 
+'''
\ No newline at end of file
